// The original version of this code (revision 2 of the Subversion
// archive) was released subject to the following license:

//   Copyright (c) 2006, Sun Microsystems, Inc.  All rights reserved.
//   Redistribution and use in source and binary forms, with or
//   without modification, are permitted provided that the following
//   conditions are met:

//   * Redistributions of source code must retain the above copyright
//     notice, this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above
//     copyright notice, this list of conditions and the following
//     disclaimer in the documentation and/or other materials provided
//     with the distribution.
//   * Neither the name of Sun Microsystems or the names of
//     contributors may be used to endorse or promote products derived
//     from this software without specific prior written permission.
 
//   THIS SOFTWARE IS PROVIDED BY SUN AND ITS LICENSORS ``AS IS'' AND
//   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
//   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
//   PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SUN OR ITS
//   LICENSORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
//   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
//   AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
//   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
//   ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
//   POSSIBILITY OF SUCH DAMAGE.

// Subsequent additions and modifications are Copyright (c) 2006 David
// Carlton, and may be used subject to the same conditions.

#ifndef UNITTEST_EXISTINGBASE_H
#define UNITTEST_EXISTINGBASE_H

#include "UnitTest.hpp"
#include <sstream>

// A base class for tests that want to make sure that classes
// construct and destroy the proper numbers of elements.  It's
// templated on a type T which is assumed to provide static methods
// existing() and resetExisting(); its destructor checks that there
// are no remaining objects at the end of the test.

// You can provide the existing() and resetExisting() methods
// automatically by having your class derive from ExistingObject<T>.

namespace UnitTest {

  template<typename T>
  class ExistingBase {
  public:
    ExistingBase() {
      T::resetExisting();
    }

    ~ExistingBase() {
      if (existing() != 0) {
	std::ostringstream oss;
	oss << "Objects remain at end of test: " << existing();
	FAIL(oss.str());
      }
    }

    int existing() const {
      return T::existing();
    }
  };

  template<typename T>
  class ExistingObject {
  public:
    static int existing() {
      return existing_;
    }

    static void resetExisting() {
      existing_ = 0;
    }

  protected:
    ExistingObject() {
      ++existing_;
    }

    ExistingObject(const ExistingObject &other) {
      ++existing_;
    }

    ~ExistingObject() {
      --existing_;
    }

  private:
    static int existing_;
  };

  template<typename T>
  int ExistingObject<T>::existing_;

}

#endif // UNITTEST_EXISTINGBASE_H
